package tdspp.DeploymentAdvisor;

import java.util.*;
import tdspp.util.*;

/**
 * Helps in deployment of the resources.
 *
 * This will be implemented as a simple interface that checks the current deployment and provides
 * the information on that.
 * 
 * <table border=0> 
 * <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td> <td> TDS++ </td>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td> <td> Optional, Variant, Active </td>
 * </table> 
 * 
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 22 Feb 2002
 * 
*/

public class DeploymentAdvisorImp extends GenericImp {

	Region[][] territory;
	Region [] regions;
	Vector deployRegion, deployLevel;
	int[][] deployPriority;
	boolean [][] visited;
	Hashtable localRules;


	public DeploymentAdvisorImp(){
		 territory = new Region[Constants.REPOSITORY_CELLS_PER_SUBMAP][Constants.REPOSITORY_CELLS_PER_SUBMAP];
		 regions = new Region [Constants.REPOSITORY_CELLS_PER_SUBMAP*Constants.REPOSITORY_CELLS_PER_SUBMAP];
		 deployPriority = new int[Constants.REPOSITORY_CELLS_PER_SUBMAP][Constants.REPOSITORY_CELLS_PER_SUBMAP];
		 visited = new boolean [Constants.REPOSITORY_CELLS_PER_SUBMAP][Constants.REPOSITORY_CELLS_PER_SUBMAP];
		 localRules = new Hashtable ();
		 deployRegion = new Vector();
		 deployLevel = new Vector();
	
	}

	public void setLocalRules(Hashtable localRules){
		this.localRules = localRules;
	}
	
	public Vector getDeployRegion(){
		return deployRegion;
	}
	
	public Vector getDeployLevel(){
		return deployLevel;
	}
	
	public void setRegions(Region[] regions){
		this.regions = (Region[]) regions.clone();
		for (int i=0; i<Constants.REPOSITORY_CELLS_PER_SUBMAP; i++)
			for (int j=0; j<Constants.REPOSITORY_CELLS_PER_SUBMAP; j++){
				territory[i][j]= regions[10*i+j];
				//System.out.println("territory set");
			}
	}
	
	
	private void checkAroundLevel2 (int x, int y, int i, int j) {
		if (x>=0)
			if (x<Constants.REPOSITORY_CELLS_PER_SUBMAP)
				if (y>=0)
					if (y<Constants.REPOSITORY_CELLS_PER_SUBMAP){
						if (territory[x][y].armyRes.isMineField ()){
							if (localRules.containsKey(MessageConstants.RULE_ARMYRESOURCES_MINEFIELD  ))
								deployPriority[i][j] += ((Integer) localRules.get(MessageConstants.RULE_ARMYRESOURCES_MINEFIELD)).intValue();	
						}
						else if (territory[x][y].armyRes.isEnemyInfantry ()){
							if (localRules.containsKey(MessageConstants.RULE_ARMYRESOURCES_ENEMYINFT   ))
								deployPriority[i][j] += ((Integer) localRules.get(MessageConstants.RULE_ARMYRESOURCES_ENEMYINFT )).intValue();	
						}
						else if (territory[x][y].armyRes.isEnemyTank ()){
							if (localRules.containsKey(MessageConstants.RULE_ARMYRESOURCES_ENEMYTANK   ))
								deployPriority[i][j] += ((Integer) localRules.get(MessageConstants.RULE_ARMYRESOURCES_ENEMYTANK )).intValue();	
						}
					}
	}
	
	
	
	/**
	 * this function checks the neighbouring position of friendly troops. If it is OK
	 * then it checks for its neighboring positions
	 */
	private void checkAroundLevel1 (int x, int y, int i, int j) {
		
		if (x>=0)
			if (x<Constants.REPOSITORY_CELLS_PER_SUBMAP)
				if (y>=0)
					if (y<Constants.REPOSITORY_CELLS_PER_SUBMAP){
						if (territory[x][y].armyRes .isInfantry () || territory[x][y].armyRes.isTank ()){}
						else if (!visited[x][y]){
							visited[x][y]=true;
							if (territory[x][y].terrainDesc .getType () == Constants.TERRAINTYPE_SWAMP){
								if (localRules.containsKey(MessageConstants.RULE_TERRAINTYPE_SWAMP ))
									deployPriority[x][y] += ((Integer) localRules.get(MessageConstants.RULE_TERRAINTYPE_SWAMP )).intValue();				
							}
							else if (territory[x][y].terrainDesc .getType () == Constants.TERRAINTYPE_RIVER){
								if (localRules.containsKey(MessageConstants.RULE_TERRAINTYPE_RIVER ))
									deployPriority[x][y] += ((Integer) localRules.get(MessageConstants.RULE_TERRAINTYPE_RIVER )).intValue();
							}
							else if (territory[x][y].terrainDesc .getType () == Constants.TERRAINTYPE_MOUNTAIN){
								if (localRules.containsKey(MessageConstants.RULE_TERRAINTYPE_MOUNTAIN ))
									deployPriority[x][y] += ((Integer) localRules.get(MessageConstants.RULE_TERRAINTYPE_MOUNTAIN )).intValue();
							}
							else if (territory[x][y].terrainDesc .getType () == Constants.TERRAINTYPE_FLAT){
								if (localRules.containsKey(MessageConstants.RULE_TERRAINTYPE_FLAT ))
									deployPriority[x][y] += ((Integer) localRules.get(MessageConstants.RULE_TERRAINTYPE_FLAT )).intValue();
							}
							checkAroundLevel2(x,y,x,y);
							checkAroundLevel2(x-1,y-1,x,y);
							checkAroundLevel2(x-1,y,x,y);
							checkAroundLevel2(x-1,y+1,x,y);
							checkAroundLevel2(x,y-1,x,y);
							checkAroundLevel2(x,y+1,x,y);
							checkAroundLevel2(x+1,y-1,x,y);
							checkAroundLevel2(x+1,y,x,y);
							checkAroundLevel2(x+1,y+1,x,y);
						}
					}
	}
	
	public void adviseDeployment(){
		int i, j;
	
		for (i=0; i<Constants.REPOSITORY_CELLS_PER_SUBMAP; i++)
			for (j=0; j<Constants.REPOSITORY_CELLS_PER_SUBMAP; j++){
				deployPriority[i][j]=50;
				visited[i][j]=false;
			}
	
		//System.out.println("Friendly troops are on:");
		for (i=0; i<Constants.REPOSITORY_CELLS_PER_SUBMAP; i++)
			for (j=0; j<Constants.REPOSITORY_CELLS_PER_SUBMAP; j++){
				if (territory[i][j].armyRes .isInfantry () || territory[i][j].armyRes.isTank ()){
					//System.out.println (i+" "+j);
					//checkTerrain(i,j);//we assume our strategy is OK or we want to be on the terrain(i,j)
					checkAroundLevel1(i-1,j-1,i,j);
					checkAroundLevel1(i-1,j,i,j);
					checkAroundLevel1(i-1,j+1,i,j);
					checkAroundLevel1(i,j-1,i,j);
					checkAroundLevel1(i,j+1,i,j);
					checkAroundLevel1(i+1,j-1,i,j);
					checkAroundLevel1(i+1,j,i,j);
					checkAroundLevel1(i+1,j+1,i,j);
					
				}
			}
	
		deployRegion = new Vector();
		deployLevel = new Vector();
		for (i=0; i<Constants.REPOSITORY_CELLS_PER_SUBMAP; i++)
			for (j=0; j<Constants.REPOSITORY_CELLS_PER_SUBMAP; j++)
				if (deployPriority[i][j] != 50){
					deployRegion.addElement(new Integer (Constants.REPOSITORY_CELLS_PER_SUBMAP*i+j));
					deployLevel.addElement(new Integer (deployPriority[i][j]));
				}
		System.out.println("size of the deployRegion is "+deployRegion.size());
	
	}
	
	
	public void adviseDeployment(Region[][] ter){
		
		int i, j;
		//System.out.println ("in");
		territory = (Region[][]) ter.clone();
		adviseDeployment();	
	}
	
	
	
	
	
	/**
	 * Analyze the current doployment and report the results.
	 * Called by: Rendering Agent
	 * 
	 * @param s	The strategy to be amalyzed
	 * @return	void
	 * 
	*/
	/*
	public void analyzeDeployment(Strategy s) {
	}
	*/
}
