package tdspp.Map;

import java.util.*;
import Prism.core.*;
import tdspp.util.*;
import java.awt.Image;

/**
 * Map component stores the map of a geographical area.
 * 
 * The map can be stored as a collection of regions or a collection of points.
 * Anyhow the consituents will store the respecive properties along with
 * the geographical information. These additional properties might be anything
 * that is useful for the system. We prefer storing maps as collection of regions as
 * that requires lesser memory.
 *
 * <table border=0> 
 * <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td> <td> TDS++ </td>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td> <td> Variant, Passive </td>
 * </table> 
 * 
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 15 Feb 2002
 * 
*/

public class MapImp  extends GenericImp {
	/**
	 * The collection of regions.
	 *
	*/
	Object[] array;
	
	/**
	 * Image of the submap
	 */
	TDSImage[] tdsImages;

	/**
	 * Constructor
	 * 
	*/
	public MapImp(AbstractImplementation parent) {
		super(parent);
		array = new Object[Constants.REPOSITORY_NUMBER_OF_SUBMAPS];
		tdsImages = new TDSImage[Constants.REPOSITORY_NUMBER_OF_SUBMAPS];
	}

	/**
	 * Provides the attributes for a particular region or a point.
	 * Called by the rendering agent
	 *
	 * @param r	Region whose attributes are needed.
	 * @see 	#setAttributes
	 * @return	Region object
	 * 
	*/
	public Region getAttributes(Region r) {
		Region[] regions = new Region [100];
		regions = (Region[]) array[r.submapId];
		for (int i = 0; i < regions.length ; i++) {
			Region reg = new Region ();
			reg = regions[i];
			if(r.x == reg.x && r.y == reg.y)
				return reg;
		}
		return null;
	}

	/**
	 * Sets the attributes of a region.
	 * Called by the rendering agent
	 * 
	 * @param r	Region whose attributes are to be changed.
	 * @see 	RenderingAgentImp#showMap
	 * @see 	#getAttributes
	 * @return	true or false
	 * 
	*/
	public boolean setAttributes(Region r) {
		Region[] regions = new Region [100];
		regions = (Region[]) array[r.submapId];
		for (int i = 0; i < regions.length ; i++) {
			Region reg = new Region ();
			reg = regions[i];
			if(r.x == reg.x && r.y == reg.y) {
				regions[i] = r;
				return true;
			}
		}
		return false;
	}

	/**
	 * Updates the regions data.
	 * Called by the Simulation Agent
	 * 
	 * @param arr Array of Regions
	 * 
	*/
	public void update(int submapId, Region[] newRegions) {
		array[submapId] = (Region[]) newRegions.clone();
		/*
		if (((Region[])array[submapId])[0].armyRes.resources == 65536) 
			System.out.println("successful");
		else System.out.println("unsuccessful");
		*/
	}

	/**
	 * sets the image data.
	 * 
	*/
	public void setTDSImage(int submapId, TDSImage newTdsImage) {
		//this.tdsImages[submapId] = (TDSImage) newTdsImage.clone(); for some reason this doesn't compile?
		this.tdsImages [submapId] = new TDSImage ();
		this.tdsImages [submapId].inputName = newTdsImage.inputName;
		this.tdsImages [submapId].outputName = newTdsImage.outputName;
		this.tdsImages [submapId].img = newTdsImage.img;
	}

	public void setAllTDSImages (TDSImage[] images){
		this.tdsImages = (TDSImage[]) images.clone();
	}
	
	/**
	 * get function to get regions for a submap
	 */
	public Region[] getRegions(int submapId) {
		return (Region[]) array[submapId];
	}
	
	/**
	 * get function to get image for a submap
	 */
	public TDSImage getTDSImage(int submapId) {
		return tdsImages[submapId];
	}
	
	/**
	 * get function to get all regions
	 */
	public Object[] getAllRegions() {
		//System.out.println ("getting all regions from Map");
		
		return array;
		
	}
	
	public void setAllRegions (Object[] array){
		//System.out.println ("Map setting all regions");
		this.array = (Object[]) array.clone();
		
	}
	
	/**
	 * get function to get all images
	 */
	public TDSImage[] getAllTDSImages() {
		return tdsImages;
	}
	
	
}
