
package tdspp.RenderingAgent;

import java.util.*;
import tdspp.util.*;
import Prism.core.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
/**
 * The main frame displayed by the RenderingAgent. The frame consists of two panels.
 * The top panel shows the map and lower panel shows the options available.
 * This is defined as a local class as it is used only by the RenderingAgent.
 */

public class TDSMainFrame extends Frame implements WindowListener {
	/**
	 * Parent Rendering Agent.
	 */
	public RenderingAgentImp parent;

	/**
	 * Currently selected cell x
	 */
	int cellX;

	/**
	 * Currently selected cell y
	 */
	int cellY;

	/**
	 * The top panel
	 */
	public TopPanel tp;

	/**
	 * The bottom panel
	 */
	public BottomPanel bp;

	/**
	 * Allowed to change current submap
	 */
	public boolean mySubmap;

	/**
	 * Constructor
	 */
	public TDSMainFrame(String name, RenderingAgentImp par) {
		super(name);
		mySubmap = false;
		parent = par;
		cellX = cellY = -1;
		setSize(tdspp.util.Constants.MAINFRAME_WIDTH,tdspp.util.Constants.MAINFRAME_HEIGHT);
		setResizable(false);
		setLayout(new BorderLayout(0,0));
		addWindowListener(this);
		tp = new TopPanel(parent,this);
		tp.setSize(tdspp.util.Constants.MAINFRAME_WIDTH,tdspp.util.Constants.MAINFRAME_WIDTH);
		add(tp,BorderLayout.CENTER);
		bp = new BottomPanel(parent,this);
		bp.setSize(tdspp.util.Constants.MAINFRAME_WIDTH,tdspp.util.Constants.MAINFRAME_HEIGHT - tdspp.util.Constants.MAINFRAME_WIDTH - tdspp.util.Constants.MAINFRAME_CAPTION_HEIGHT);
		add(bp,BorderLayout.SOUTH);
		doLayout();
		show();
	}

	/**
	 * Window closing function
	 */
	public void windowClosing(WindowEvent e) {
		dispose();
	}

	/**
	 * Window activated
	 */
	public void windowActivated(WindowEvent e) {
	}

	/**
	 * Window closed
	 */
	public void windowClosed(WindowEvent e) {
		System.exit(0);
	}

	/**
	 * Window deactivated
	 */
	public void windowDeactivated(WindowEvent e) {
	}

	/**
	 * Window iconified
	 */
	public void windowIconified(WindowEvent e) {
	}

	/**
	 * Window opened
	 */
	public void windowOpened(WindowEvent e) {
	}

	/**
	 * Window deiconified
	 */
	public void windowDeiconified(WindowEvent e) {
	}

	/**
	 * Check if the current submap is mine
	 */
	public void checkSubmapMine() {
		mySubmap = false;
		if(parent.mySubmaps == null) return;
		for(int i = 0; i < parent.mySubmaps.length; i++)
			if(parent.mySubmaps[i] == parent.submapId) {
				mySubmap = true;
				return;
			}
	}


	/**
	 * Check if the current submap is mine
	 */
	public boolean checkSubmapMine(int submap) {
		if(parent.mySubmaps == null) return false;
		for(int i = 0; i < parent.mySubmaps.length; i++) {
			if(parent.mySubmaps[i] == submap) {
				return true;
			}
		}
		return false;
	}

}
