package tdspp.Repository;

import java.util.*;
import tdspp.util.*;
import java.io.*;
import Prism.core.*;
import java.awt.Image;

/**
 * This is the main information repository in the system. Maintains information pertaining to a
 * particular execution of the system.
 *
 * It is implemented as a component that opens a log file and logs all the events to the same file.
 *
 * <table border=0>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td> <td> TDS++ </td>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td> <td> Required, Passive </td>
 * </table>
 *
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 22 Feb 2002
 *
*/

public class RepositoryImp extends GenericImp {
	/**
	 * The files which are used for data storage.
	 *
	*/
	File file, tempFile;



	/**
	 * Region data : each element is itself an array of Region objects.
	 * Each element in turn has 10x10 objects arranged in row-major order.
	 */
	Object[] array;

	/**
	 * The array of images
	 */
	TDSImage[] tdsImages;

	/**
	 * The array of SAKB factors
	 */
	String[] factors;

	/**
	 * The array of SAKB values
	 */
	Integer[] values;

	public int factNum = 0;



	/**
	 * Constructor
	 *
	*/
	public RepositoryImp(AbstractImplementation parent) {
		super(parent);
		tdsImages = new TDSImage[Constants.REPOSITORY_NUMBER_OF_SUBMAPS];
		array = null;
		factors = new String [10];
		values = new Integer [10];
	}

	/**
	 * Updates the regions data.
	 *
	 * @param arr Array of Regions
	 *
	*/
	public void update(int submapId, Region[] newRegions) {
		array[submapId] = newRegions;
	}

	/**
	 * get function to get regions for a submap
	 */
	public Region[] getRegions(int submapId) {
		return (Region[]) array[submapId];
	}

	/**
	 * get function to get image for a submap
	 */
	public TDSImage getTDSImage(int submapId) {
		return tdsImages[submapId];
	}

	/**
	 * get function to get factors
	 */
	public String[] getFactors() {
		return factors;
	}
	public void setFactors(String[] factors){
		this.factors = (String[]) factors.clone();
	}

	/**
	 * get function to get values
	 */
	public Integer[] getValues() {
		return values;
	}
	public void setValues (Integer[] values){
		this.values = (Integer[]) values.clone();
	}

	/**
	 * get function to get all regions
	 */
	public Object[] getAllRegions() {
		return array;
	}

	public void setAllRegions (Object[] array){

		this.array = (Object[]) array.clone();

	}

	/**
	 * get function to get all images
	 */
	public TDSImage[] getAllTDSImages() {
		return tdsImages;
	}

	/**
	 * Write the current state into the file
	 */
	public void writeFile(String filename){
		try {

			tempFile = new File(Constants.REPOSITORY_TEMP_OUTPUT_FILENAME );
			PrintWriter writer = new PrintWriter(new FileWriter(file));
			BufferedReader tempReader = new BufferedReader(new FileReader(tempFile));
			String line, tempLine;

			//Rules
			line=";Rules";
			writer.println(line);
			//System.out.println ("factNum = "+factNum);
			for (int i=0; i<factNum; i++) {
				line = ":rule "+factors[i]+" "+values[i].toString();
				writer.println(line);
			}

			//Images
			writer.println(";Images\n; filenames of the images for the submaps");
			while ((tempLine = tempReader.readLine()) != null) {
				writer.println(tempLine);
			}


			//Region Data
			writer.println(";\n; region data");
			// cx cy submapid temperature humidity windSpeed sunlight resources stateAmmu stateFood statePop terraintype
			for (int i=0; i<Constants.REPOSITORY_NUMBER_OF_SUBMAPS ; i++){
				Region[] reg = (Region[]) array[i];
				//if (reg == null) System.out.println("array["+i+"] is null");
				for (int j=0; j<Constants.REPOSITORY_CELLS_PER_SUBMAP ; j++){
					for (int k=0; k<Constants.REPOSITORY_CELLS_PER_SUBMAP ;k++){
						//if (reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k] == null) System.out.println("reg is null");
						writer.print(reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].x+" ");
						writer.print(reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].y+" ");
						writer.print(reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].submapId+" ");
						if (reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].weatherInfo != null){
							writer.print(reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].weatherInfo.temperature+" ");
							writer.print(reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].weatherInfo.humidity+" ");
							writer.print(reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].weatherInfo.windSpeed+" ");
							writer.print(reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].weatherInfo.sunlight+" ");
						} else writer.print("0.0 0.0 0.0 0 ");
						if (reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].armyRes != null){
							writer.print(reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].armyRes.resources+" ");
							writer.print(reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].armyRes.stateAmmunition+" ");
							writer.print(reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].armyRes.stateFood+" ");
							writer.print(reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].armyRes.statePopulation+" ");
						} else writer.print("0 0.0 0.0 0.0");
						if (reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].terrainDesc != null){
							writer.print(reg[j*Constants.REPOSITORY_CELLS_PER_SUBMAP + k].terrainDesc.type+" ");
						} else writer.print ("0");
						writer.println();
					}
				}
			}
			writer.close();
			tempReader.close();
		}
		catch (Exception e) {
			System.out.println("Repository.writeFile : " + e);
		}
	}

	/**
	 * Read the file and read the initial data
	 */
	public void readFile(String filename) {
		try {
			file = new File(filename);
			BufferedReader br = new BufferedReader(new FileReader(file));
			tempFile = new File(Constants.REPOSITORY_TEMP_OUTPUT_FILENAME );
			PrintWriter tempWriter = new PrintWriter(new FileWriter(tempFile));
			String line;
			int lineNum = 0;

			if(array == null) {
				array = new Object[Constants.REPOSITORY_NUMBER_OF_SUBMAPS];
				for(int i = 0; i < Constants.REPOSITORY_NUMBER_OF_SUBMAPS; i++) {
					Region[] subarr = new Region[Constants.REPOSITORY_CELLS_PER_SUBMAP * Constants.REPOSITORY_CELLS_PER_SUBMAP];
					for(int j = 0; j < Constants.REPOSITORY_CELLS_PER_SUBMAP * Constants.REPOSITORY_CELLS_PER_SUBMAP; j++) {
						subarr[j] = new Region();
						subarr[j].submapId = i;
						subarr[j].x = j / Constants.REPOSITORY_CELLS_PER_SUBMAP;
						subarr[j].y = j % Constants.REPOSITORY_CELLS_PER_SUBMAP;
					}
					array[i] = subarr;
				}
			}
			while((line = br.readLine()) != null) {
				line = line.trim();
				if(line.charAt(0) == ';') continue;
				//rules are at the beginning of the file, denoted by ':'
				if (line.charAt(0) == ':'){//each rule has factor and value
					StringTokenizer st = new StringTokenizer(line);
					int tokens = st.countTokens();
					if (tokens > 2){
						//System.out.println("successful");
						st.nextToken();
						String tempFactor = st.nextToken();
						int tempValue = Integer.parseInt(st.nextToken());
						factors [factNum] = tempFactor;
						values [factNum] = new Integer (tempValue);
						factNum++;
						//System.out.println(tempFactor + " "+ tempValue);
					}
				}
				else {
					if(lineNum < Constants.REPOSITORY_NUMBER_OF_SUBMAPS) {
						// reading image names
						//System.out.println("Repository.readFile : Image[" + lineNum + "] " + line);
						tdsImages[lineNum]= new TDSImage ();
						tdsImages[lineNum].inputName = new String(line);
						tdsImages[lineNum].outputName = new String("t"+line.substring(4,line.length()));
						
						tdsImages[lineNum].img = java.awt.Toolkit.getDefaultToolkit().getImage(line);
						tempWriter.println(line);
						if(tdsImages[lineNum].img == null)
							System.out.println("Repository.readFile : Error loading the image " + line);
						lineNum++;
					}
					else {
						// reading Region data
						//System.out.println("Repository.readFile : Region " + line);
						// cx cy submapid temperature humidity windSpeed sunlight resources stateAmmu stateFood statePop terraintype
						StringTokenizer st = new StringTokenizer(line);
						int tokens = st.countTokens();
						if(tokens > 7) {
							// atleast 8 tokens required
							Region r = new Region();
							r.x = Integer.parseInt(st.nextToken());
							tokens--;
							r.y = Integer.parseInt(st.nextToken());
							tokens--;
							r.submapId = Integer.parseInt(st.nextToken());
							tokens--;
							r.weatherInfo = new WeatherInfo();
							r.armyRes = new ArmyResources();
							r.terrainDesc = new TerrainDesc();

							//st.nextToken(); // temperature
							r.weatherInfo.temperature = new Float(st.nextToken()).floatValue();
							tokens--;
							//st.nextToken(); // humidity
							r.weatherInfo.humidity = new Float(st.nextToken()).floatValue();
							tokens--;
							//st.nextToken(); // windSpeed
							r.weatherInfo.windSpeed = new Float(st.nextToken()).floatValue();
							tokens--;
							//st.nextToken(); // sunlight
							r.weatherInfo.sunlight = Integer.parseInt(st.nextToken());
							tokens--;
							if(tokens-- > 0) r.armyRes.resources = Integer.parseInt(st.nextToken());
							if(tokens-- > 0) r.armyRes.stateAmmunition = new Float(st.nextToken()).floatValue();
							if(tokens-- > 0) r.armyRes.stateFood = new Float(st.nextToken()).floatValue();
							if(tokens-- > 0) r.armyRes.statePopulation = new Float(st.nextToken()).floatValue();

							//System.out.println("total = "+r.armyRes.getTotalState());
							if(tokens-- > 0) r.terrainDesc.type = Integer.parseInt(st.nextToken());
							setAttributes(r);
						}
					}
				}
			}
			br.close();
			tempWriter.close();
		} catch(Exception e) {
			System.out.println("Repository.readFile : " + e);
		}
	}

	/**
	 * Sets the attributes of a region.
	 *
	 * @param r	Region whose attributes are to be changed.
	 * @return	true or false
	 *
	*/
	public boolean setAttributes(Region r) {
		Region[] regions = (Region[]) array[r.submapId];
		for (int i = 0; i < regions.length ; i++) {
			Region reg = regions[i];
			if(r.x == reg.x && r.y == reg.y) {
				regions[i] = r;
				return true;
			}
		}
		return false;
	}

	/**
	 * Logs a particular event into the repository or the log file.
	 * Called by: Any component
	 *
	 * @param ev	The event object
	 * @see 	#checkEvent
	 * @return	void
	 *
	*/
	public void logEvent(
						 // Event ev
						 ) {
	}

	/**
	 * Dumps the log into a file.
	 * Called by: Any component
	 *
	 * @see 	#logState
	 * @return	void
	 *
	*/
	public void dumpLog() {
	}

	/**
	 * Checks for the existence of the given event in the log.
	 * Called by: Any component
	 *
	 * @param ev	The event object
	 * @see 	#logEvent
	 * @return	void
	 *
	*/
	public void checkEvent(
						   // Event ev
						   ) {
	}

	/**
	 * Logs the state of the system onto a file. The optional filename gives the name of the file
	 * to be written to.
	 * Called by: Any component
	 *
	 * @param file	The name of the file to be produced.
	 * @see 	#retrState
	 * @return	void
	 *
	*/
	public void logState(String filename) {
	}

	/**
	 * Retrives the state of the system from a given filename. The optional component
	 * argument might be helpful if we wish to retrieve the state of a particular component only.
	 * Called by: Any component
	 *
	 * @param ev	Name of the file to be read information from.
	 * @param c	Component to retrieve the state for.
	 * @see 	#logState
	 * @return	void
	 *
	*/
	public void retrState(String filename, Component comp) {
	}
}
