package tdspp.ResourceManager;

import Prism.core.*;
import tdspp.util.*;

/**
 * The resource manager component. Makes use of class ResourceManagerImp.
 *
 * @see		ResourceManagerImp
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 18 Feb 2002
 *
*/

public class ResourceManager extends AbstractImplementation {
	/**
	 * The ResourceManagerImp object
	 *
	 */
	ResourceManagerImp resourceManagerIO;
	private static tdspp.util.MessageConstants mc;

	/**
	 * Default constructor
	 */
	public ResourceManager() {
		this("ResourceManager");
	}

	/**
	 * Constructor
	 *
	 */
	public ResourceManager(String name) {
		if(resourceManagerIO == null) {
			resourceManagerIO = new ResourceManagerImp(this);
		}
	}

	/**
	 * Handling the Requests
	 *
	 * @param r	Request
	 * @return 	void
	 *
	 */
	public void handle(Event r) {
//		System.out.println("Resource Manager Request >>> " + r.name);
		if(r.name.equalsIgnoreCase(mc.MESSAGE_NAME_GETRESOURCES)) {

			//send request to Map for Region object
			Event newr = new Event(mc.MESSAGE_NAME_GETATTR);
			newr.eventType=PrismConstants.REQUEST;
			newr.addParameter(mc.PARAM_NAME_GETATTR_REGION, (Region)r.getParameter(mc.PARAM_NAME_GETRESOURCES_REGION));
			send(newr);
			//return true;
		}

		else if(r.name.equalsIgnoreCase(mc.MESSAGE_NAME_ADDRESOURCES)) {

			Region reg = (Region)r.getParameter(mc.PARAM_NAME_ADDRESOURCES_REGION);
			int opFlag = 1;
			String resourceNames[];

			resourceNames = (String [])r.getParameter(mc.PARAM_NAME_ADDRESOURCES_RESOURCENAMES);
	//		System.out.println("Setting atttr/////////////// " + opFlag);
			resourceManagerIO.setAttributes(resourceNames, opFlag, reg);
	//		System.out.println("Set atttr///////////////");
			//send request to Map for Region object
			Event newr = new Event(mc.MESSAGE_NAME_GETATTR);
			newr.eventType=PrismConstants.REQUEST;
			newr.addParameter(mc.PARAM_NAME_GETATTR_REGION, reg);
			send(newr);
			//return true;
		}


		else if(r.name.equalsIgnoreCase(mc.MESSAGE_NAME_DELETERESOURCES)) {
			Region reg = (Region)r.getParameter(mc.PARAM_NAME_DELETERESOURCES_REGION);
			String resourceNames[];
			int opFlag = -1;

			resourceNames = (String[])r.getParameter(mc.PARAM_NAME_DELETERESOURCES_RESOURCENAMES);

			resourceManagerIO.setAttributes(resourceNames, opFlag, reg);

			//send request to Map for Region object
			Event newr = new Event(mc.MESSAGE_NAME_GETATTR);
			newr.eventType=PrismConstants.REQUEST;
			newr.addParameter(mc.PARAM_NAME_GETATTR_REGION, reg);
			send(newr);
			//return true;
		}

		else if(r.name.equalsIgnoreCase(mc.MESSAGE_NAME_MOVERESOURCES)) {
			Region fromreg = (Region)r.getParameter(mc.PARAM_NAME_MOVERESOURCES_FROMREGION);
			Region toreg = (Region)r.getParameter(mc.PARAM_NAME_MOVERESOURCES_TOREGION);

			String resourceNames[];
			int opFlag = 2;


			resourceNames = (String [])r.getParameter(mc.PARAM_NAME_MOVERESOURCES_RESOURCENAMES);
			resourceManagerIO.setAttributes(resourceNames, opFlag, fromreg, toreg);

			//send request to Map for Region object ....
			Event newr = new Event(mc.MESSAGE_NAME_GETATTR);
			newr.eventType=PrismConstants.REQUEST;
			newr.addParameter(mc.PARAM_NAME_GETATTR_REGION, fromreg);
			send(newr);

			newr = new Event(mc.MESSAGE_NAME_GETATTR);
			newr.eventType=PrismConstants.REQUEST;
			newr.addParameter(mc.PARAM_NAME_GETATTR_REGION, toreg);
			send(newr);
			//return true;
		}

		else if(r.name.equalsIgnoreCase(mc.MESSAGE_NAME_SUPPLYRESOURCES)) {
			//System.out.println("Supply got request ........ " + mc.MESSAGE_NAME_SUPPLYRESOURCES);

			Region reg = new Region();
			reg.x = ((Integer)r.getParameter(mc.PARAM_NAME_SUPPLYRESOURCES_XCOOR)).intValue();
			reg.y = 0;
			resourceManagerIO.setAttributes(3, reg);

			//System.out.println("X OCOR........ " + reg.x);

			//System.out.println("Supply Getting map ........ ");
			Event newr = new Event(mc.MESSAGE_NAME_GETMAPREGIONS);
			newr.eventType=PrismConstants.REQUEST;
			newr.addParameter(mc.PARAM_NAME_GETMAP_SUBMAPID, (Integer)r.getParameter (mc.PARAM_NAME_GETMAP_SUBMAPID));
			send(newr);
			//return true;
		}

		else if(r.name.equalsIgnoreCase(mc.MESSAGE_NAME_DEPLETERESOURCES)) {

			Region reg = new Region();
			//System.out.println("Deplete got request ........ " + mc.MESSAGE_NAME_SUPPLYRESOURCES);
			reg.x = ((Integer)r.getParameter(mc.PARAM_NAME_DEPLETERESOURCES_XCOOR)).intValue();
			reg.y = 0;
			resourceManagerIO.setAttributes(4, reg);
			//System.out.println("X OCOR........ " + reg.x);
			//System.out.println("Deplete Getting map ........ ");
			Event newr = new Event(mc.MESSAGE_NAME_GETMAPREGIONS );
			newr.eventType=PrismConstants.REQUEST;
			newr.addParameter(mc.PARAM_NAME_GETMAP_SUBMAPID, (Integer)r.getParameter (mc.PARAM_NAME_GETMAP_SUBMAPID));
			send(newr);
			//return true;
		}
		else handleN(r);
	}



	/**
	 * Handling the Notifications
	 *
	 * @param r	Notification
	 * @return 	void
	 *
	 */
	public void handleN (Event n) {
		/*if(n.name.equalsIgnoreCase(mc.MESSAGE_NAME_GETATTRRESP))
			System.out.println("Resource Manager notification >>> " + n.name);
		*/
		if(n.name.equalsIgnoreCase(mc.MESSAGE_NAME_GETATTRRESP)) {
			doUpdate(n);
			//return true;
		}

		if(n.name.equalsIgnoreCase(mc.MESSAGE_NAME_GETMAPREGIONSRESP)) {
			int submapId = ((Integer) n.getParameter(mc.PARAM_NAME_GETMAP_SUBMAPID)).intValue();
			//System.out.println("resourceManagerIO.getPeriodicFlag() >> " + resourceManagerIO.getPeriodicFlag());
			if(resourceManagerIO.getPeriodicFlag() == 3) {			//supply resources
				Region regArray[];
				regArray = (Region [])n.getParameter(mc.PARAM_NAME_GETMAPRESP_REGIONS);
				Event r = new Event(mc.MESSAGE_NAME_SETMAP);
				r.eventType=PrismConstants.REQUEST;
				r.addParameter(mc.PARAM_NAME_SETMAP_SUBMAPID, new Integer(submapId));
				r.addParameter(mc.PARAM_NAME_SETMAP_REGIONS, (Region [])resourceManagerIO.supplyResources(regArray));
				send(r);
			}
			else if (resourceManagerIO.getPeriodicFlag() == 4) {		//deplete resources
				Region regArray[];
				regArray = (Region [])n.getParameter(mc.PARAM_NAME_GETMAPRESP_REGIONS);
				Event r = new Event(mc.MESSAGE_NAME_SETMAP);
				r.eventType=PrismConstants.REQUEST;
				r.addParameter(mc.PARAM_NAME_SETMAP_SUBMAPID, new Integer(submapId));
				r.addParameter(mc.PARAM_NAME_SETMAP_REGIONS, (Region [])resourceManagerIO.depleteResources(regArray));
				send(r);
			}
			//return true;
		}
		//return false;
	}

	private synchronized void doUpdate(Event n){
		//System.out.println("resourceManagerIO.getOpFlag() >> " + resourceManagerIO.getOpFlag());
		if(resourceManagerIO.getOpFlag() == 0) { //0 -> get
			Region reg = (Region)n.getParameter(mc.PARAM_NAME_GETATTRRESP_REGION);
			Event newn = new Event(mc.MESSAGE_NAME_GETRESOURCESRESP);
			newn.eventType=PrismConstants.REPLY;
			newn.addParameter(mc.PARAM_NAME_GETRESOURCESRESP_ARMYRESOURCES,(ArmyResources)reg.armyRes);
			send(newn);
		}
		else {
			Region reg = (Region)n.getParameter(mc.PARAM_NAME_GETATTRRESP_REGION);
			//System.out.println("xalling upfare.......");
			Region newreg = resourceManagerIO.updateResources(reg);
			Event r = new Event(mc.MESSAGE_NAME_SETATTR);
			r.eventType=PrismConstants.REQUEST;
			r.addParameter(mc.PARAM_NAME_SETATTR_REGION, newreg);
			send(r);
		}
	}

}
