package tdspp.ResourceMonitor;

import Prism.core.*;
import tdspp.util.*;

/**
 * The resource monitor component. Makes use of class ResourceMonitorImp.
 *
 * @see		ResourceMonitorImp
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 18 Feb 2002
 *
*/

public class ResourceMonitor extends AbstractImplementation {
	/**
	 * The ResourceMonitorImp object
	 *
	 */
	ResourceMonitorImp resourceMonitorIO;

	/**
	 * Reference to Message Constants
	 *
	 */
	private static tdspp.util.MessageConstants mc;

	/**
	 * Constructor
	 *
	 */
	public ResourceMonitor(String name) {
		;
		if(resourceMonitorIO == null) {
			resourceMonitorIO = new ResourceMonitorImp(this);
		}
	}

	public ResourceMonitor (){
		this ("ResourceMonitor");
	}

	/**
	 * Handling the Requests
	 *
	 * @param r	Request
	 * @return 	void
	 *
	 */
	public void handle (Event r) {
		//System.out.println("Resource Monitor Request >>> " + r.name);
		if(r.name.equalsIgnoreCase(mc.MESSAGE_NAME_FETCHRESOURCES)) {
			//send request to Map for Region object
			Event newr = new Event(mc.MESSAGE_NAME_GETRESOURCES);
			newr.eventType=PrismConstants.REQUEST;
			newr.addParameter(mc.PARAM_NAME_GETRESOURCES_REGION, (Region)r.getParameter(mc.PARAM_NAME_FETCHRESOURCES_REGION));
			send(newr);
		} else if(r.name.equalsIgnoreCase(mc.MESSAGE_NAME_UPDATERESOURCES)) {
			Event newr;

			int opFlag = ((Integer)r.getParameter(mc.PARAM_NAME_UPDATERESOURCES_OPERATIONFLAG)).intValue();
			//System.out.println("opFlag = "+ opFlag);

			if(opFlag == 1) {		//Add resources to region

				newr = new Event(mc.MESSAGE_NAME_ADDRESOURCES);
				newr.eventType=PrismConstants.REQUEST;
				newr.addParameter(mc.PARAM_NAME_ADDRESOURCES_REGION,(Region)r.getParameter(mc.PARAM_NAME_UPDATERESOURCES_REGION));
				newr.addParameter(mc.PARAM_NAME_ADDRESOURCES_RESOURCENAMES,(String[])r.getParameter(mc.PARAM_NAME_UPDATERESOURCES_RESOURCENAMES ));
				send(newr);
			}

			if(opFlag == -1) {		//Delete resources from region
				newr = new Event(mc.MESSAGE_NAME_DELETERESOURCES);
				newr.eventType=PrismConstants.REQUEST;
				newr.addParameter(mc.PARAM_NAME_DELETERESOURCES_REGION,(Region)r.getParameter(mc.PARAM_NAME_UPDATERESOURCES_REGION));
				newr.addParameter(mc.PARAM_NAME_DELETERESOURCES_RESOURCENAMES,(String[])r.getParameter(mc.PARAM_NAME_UPDATERESOURCES_RESOURCENAMES ));
				send(newr);
			}

			if(opFlag == 2) {		//Move resources from region
				newr = new Event(mc.MESSAGE_NAME_MOVERESOURCES);
				newr.eventType=PrismConstants.REQUEST;
				newr.addParameter(mc.PARAM_NAME_MOVERESOURCES_FROMREGION ,(Region)r.getParameter(mc.PARAM_NAME_UPDATERESOURCES_FROMREGION));
				newr.addParameter(mc.PARAM_NAME_MOVERESOURCES_TOREGION ,(Region)r.getParameter(mc.PARAM_NAME_UPDATERESOURCES_TOREGION));
				newr.addParameter(mc.PARAM_NAME_MOVERESOURCES_RESOURCENAMES,(String[])r.getParameter(mc.PARAM_NAME_UPDATERESOURCES_RESOURCENAMES));
				send(newr);
			}
		} else if (r.name.equals(mc.MESSAGE_NAME_GETMAP )) {
			resourceMonitorIO.submapId = ((Integer) r.getParameter(tdspp.util.MessageConstants.PARAM_NAME_GETMAP_SUBMAPID)).intValue();
		}
		else handleN(r);
	}

	/**
	 * Handling the Notifications
	 *
	 * @param r	Notification
	 * @return 	void
	 *
	 */
	public void handleN (Event n) {
		//System.out.println("Resource Monitor Notification >>> " + n.name);
		if(n.name.equalsIgnoreCase(mc.MESSAGE_NAME_TICK)) {		//Clock Tick

			resourceMonitorIO.tick();
			if(resourceMonitorIO.deplete()) {			//If 20 Clock ticks up, request deplete resource
				Event r = new Event(mc.MESSAGE_NAME_DEPLETERESOURCES);
				r.eventType=PrismConstants.REQUEST;
				r.addParameter (mc.PARAM_NAME_GETMAP_SUBMAPID , new Integer (resourceMonitorIO.submapId));
				r.addParameter(mc.PARAM_NAME_DEPLETERESOURCES_XCOOR,new Integer(resourceMonitorIO.getXDepleted()));
				send(r);
		//		System.out.println("Sent deplete >>> ");
			}
			if(resourceMonitorIO.supply()) {		//If 100 Clock ticks up, request supply resource
				Event r = new Event(mc.MESSAGE_NAME_SUPPLYRESOURCES);
				r.eventType=PrismConstants.REQUEST;
				r.addParameter (mc.PARAM_NAME_GETMAP_SUBMAPID , new Integer (resourceMonitorIO.submapId));
				r.addParameter(mc.PARAM_NAME_SUPPLYRESOURCES_XCOOR,new Integer(resourceMonitorIO.getXSupplied()));
				send(r);
		//			System.out.println("Sent supply >>> " + resourceMonitorIO.getX());
			}
			//return false;
		}
		if(n.name.equalsIgnoreCase(mc.MESSAGE_NAME_GETRESOURCESRESP)) {		//Response to GETRESOURCE request
																			//Send Army Resource to UI
			Event newn = new Event(mc.MESSAGE_NAME_FETCHRESOURCESRESP);
			newn.eventType=PrismConstants.REPLY;
			newn.addParameter(mc.PARAM_NAME_GETRESOURCESRESP_ARMYRESOURCES,(ArmyResources)n.getParameter(mc.PARAM_NAME_FETCHRESOURCESRESP_ARMYRESOURCES));
			send(newn);
			//return true;
		}
		//return false;
	}
}
