package tdspp.ResourceMonitor;

import Prism.core.*;
import tdspp.util.*;
import java.util.*;


/**
 * This component monitors the usage and utilization of the resources.
 *
 * This component works in collaboration with the ResourceManager and updates the status
 * of the resources in that.
 *
 * <table border=0>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td> <td> TDS++ </td>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td> <td> Optional, Variant, Active </td>
 * </table>
 *
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 22 Feb 2002
 *
*/

public class ResourceMonitorImp extends GenericImp {


	/**
	 *	Count ticks before sending deplete message
	**/
	private int depleteTimeLeft;

	/**
	 *	Count ticks before sending supply message
	**/
	private int supplyTimeLeft;

	/**
	 *	X-Coordinate of SubMap to be depleted resource
	**/
	private int XCoorDepleted;

	/**
	 *	X-Coordinate of SubMap to be supplied
	**/
	private int XCoorSupplied;

	/**
	 * Index of submap currently being displayed
	 */
	public int submapId;

	
	/**
	 * Constructor
	 *
	*/
	public ResourceMonitorImp(AbstractImplementation parent) {
		super(parent);
		depleteTimeLeft = Constants.DEPLETE_TIME_LEFT ;
		supplyTimeLeft = Constants.SUPPLY_TIME_LEFT ;
		XCoorDepleted = -1;
		XCoorSupplied = -1;
		submapId = 0;
	}

	/**
	 * Update the data for a particular type of resource.
	 * Called by: Rendering Agent
	 *
	 * It might take some information from the clock and do periodic updates.
	 *
	 * @param res	Resources handle or set
	 * @see 	#fetchResources
	 * @see 	ResourceManagerImp#availableResources
	 * @return	void
	 *
	*/
	public void updateResources(int res) {
	}

	/**
	 * Fetch the data for a particular type of resource.
	 * Called by: Rendering Agent
	 *
	 * @param res	Resources handle or set
	 * @see 	#updateResources
	 * @see 	ResourceManagerImp#availableResources
	 * @return	void
	 *
	*/
	public int fetchResources(int res) {
		return res;
	}

	/**
	 * Alert the system of the state of the resources.
	 * Called by: Rendering Agent
	 *
	 * @param c	Component to alert.
	 * @return	void
	 *
	*/
	public void alertComponent(Component com) {

		//To DO !!
	}

	/**
	 * Returns X-Coordinate of SubMap to be depleted resource
	 *
	 * @param void
	 * @return	int	X-Coordinate of SubMap to be depleted resource
	 *
	*/

	public synchronized int getXDepleted (){
		if(XCoorDepleted == tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP - 1)
			XCoorDepleted = 0;
		else
			XCoorDepleted ++;
		return XCoorDepleted ;
	}


	/**
	 * Returns X-Coordinate of SubMap to be supplied resource
	 *
	 * @param void
	 * @return	int	X-Coordinate of SubMap to be supplied resource
	 *
	*/

	public synchronized int getXSupplied (){
		if(XCoorSupplied == tdspp.util.Constants.REPOSITORY_CELLS_PER_SUBMAP - 1)
			XCoorSupplied = 0;
		else
			XCoorSupplied++;
		return XCoorSupplied;
	}


	/**
	 * Count ticks
	 *
	 * @param void
	 * @return	void
	 *
	*/

	public void tick(){
		depleteTimeLeft--;
		supplyTimeLeft--;
	}


	/**
	 * Checks if it is time to send deplete request
	 *
	 * @param void
	 * @return	void
	 *
	*/

	public boolean deplete(){
		if(depleteTimeLeft == 0) {
			depleteTimeLeft = Constants.DEPLETE_TIME_LEFT;
			return true;
		}
		return false;
	}

	/**
	 * Checks if it is time to send supply request
	 *
	 * @param void
	 * @return	void
	 *
	*/

	public boolean supply(){
		if(supplyTimeLeft == 0) {
			supplyTimeLeft = Constants.SUPPLY_TIME_LEFT;
			return true;
		}
		return false;
	}
}
