package tdspp.SimulationAgent;

import Prism.core.*;
import tdspp.util.*;
import java.util .*;

/**
 * The simulation agent component. Makes use of class SimulationAgentImp.
 *
 * @see		SimulationAgentImp
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 18 Feb 2002
 *
*/

public class SimulationAgent extends AbstractImplementation {
	/**
	 * The SimulationAgentImp object
	 *
	 */
	SimulationAgentImp simaIO;
	String name;
	boolean simulationActive = false;
	int ticks = 0;


	/**
	 * Default constructor
	 */
	public SimulationAgent() {
		this("SimulationAgent");
	}

	/**
	 * Constructor
	 *
	 */
	public SimulationAgent(String name) {
		simaIO = new SimulationAgentImp();
		this.name = name;

	}

	/**
	 * Handling the Requests
	 *
	 * @param r	Request
	 * @return 	void
	 *
	 */
	public void handle (Event r) {
		if (r.name.equals (tdspp.util.MessageConstants.MESSAGE_NAME_SIMULATEFIGHT )){
			//System.out.println("simfight received");
			Region [] reg = (Region[]) r.getParameter (tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_REGIONS );
			simaIO.setRegions (reg);
			simaIO.saveRegions (reg);
			//SAKB communication
			Event rr = new Event (tdspp.util.MessageConstants.MESSAGE_NAME_GETRULES );
			rr.eventType=PrismConstants.REQUEST;
			rr.addParameter (tdspp.util.MessageConstants.PARAM_NAME_ID , new String (name));
			send (rr);
		}
		else if (r.name.equals (tdspp.util.MessageConstants.MESSAGE_NAME_DONTSAVEFIGHT )){
			Region[] oldReg = simaIO.getPrevRegions ();
			simaIO.setRegions (oldReg);
			Event nn = new Event (tdspp.util.MessageConstants.MESSAGE_NAME_DONTSAVEFIGHTRESP );
			nn.eventType=PrismConstants.REPLY;
			nn.addParameter (tdspp.util.MessageConstants.PARAM_NAME_GETMAPRESP_REGIONS, oldReg);
			send (nn);
			Event rr = new Event (tdspp.util .MessageConstants .MESSAGE_NAME_SETMAP );
			rr.eventType=PrismConstants.REQUEST;
			rr.addParameter (tdspp.util.MessageConstants.PARAM_NAME_SETMAP_REGIONS  , oldReg);
			rr.addParameter (tdspp.util .MessageConstants .PARAM_NAME_SETMAP_SUBMAPID , new Integer (oldReg[0].submapId));
			send (rr);
		}
		else handleN(r);
	}

	/**
	 * Handling the Notifications
	 *
	 * @param r	Notification
	 * @return 	void
	 *
	 */
	public void handleN (Event n) {
		if (n.name.equals (tdspp.util.MessageConstants.MESSAGE_NAME_GETRULESRESP)){
			String idName = (String) n.getParameter (tdspp.util.MessageConstants.PARAM_NAME_ID );
			if (idName.equals(name)){
				simaIO.setLocalRules ((Hashtable) n.getParameter (tdspp.util.MessageConstants.PARAM_NAME_GETRULESRESP ));
				simulationActive=true;
			}
		}
		else if (n.name.equals (tdspp.util.MessageConstants.MESSAGE_NAME_TICK )){
			//System.out.println("TICK");
			if (simulationActive){
				simaIO.simulateFight ();
				if (ticks < 10*(1000/tdspp.util.Constants .CLOCK_DELAY )){
					ticks ++;
					Event nn = new Event (tdspp.util.MessageConstants.MESSAGE_NAME_SIMULATEFIGHTRESP );
					nn.eventType=PrismConstants.REPLY;
					Region [] newRegions = simaIO.getRegions();
					Region [] clonedRegions = (Region[]) newRegions.clone();
					//System.out.println("\nSimAgent: newRegions[24].armyRes.resources = " + newRegions[24].armyRes.resources);
					nn.addParameter (tdspp.util.MessageConstants.PARAM_NAME_GETATTR_REGION , clonedRegions);
					send (nn);
				}
				else {
					simulationActive = false;
					ticks = 0;
					Region[] reg = simaIO.getRegions();
					Region[] clonedRegions = new Region [100];
					clonedRegions = (Region[]) reg.clone();
					Event nn = new Event (tdspp.util.MessageConstants.MESSAGE_NAME_SIMULATEFIGHTRESPFINAL );
					nn.eventType=PrismConstants.REPLY;
					nn.addParameter (tdspp.util.MessageConstants.PARAM_NAME_GETATTR_REGION , clonedRegions);
					//System.out.println("\nSimAgent:FINAL STEP clonedRegions[24].armyRes.resources = " + clonedRegions[24].armyRes.resources);
					send (nn);

					//System.out.println("sendingSETMAP: clonedRegions[24].armyRes.resources = " + clonedRegions[24].armyRes.resources);
					Event r = new Event (tdspp.util .MessageConstants .MESSAGE_NAME_SETMAP );
					r.eventType=PrismConstants.REQUEST;
					r.addParameter (tdspp.util .MessageConstants .PARAM_NAME_SETMAP_SUBMAPID , new Integer (reg[0].submapId));
					r.addParameter (tdspp.util.MessageConstants.PARAM_NAME_SETMAP_REGIONS  , clonedRegions);
					send (r);

				}
			}
		}
	}
}
