package tdspp.StrategyAnalysisKB;
import java.util .*;
import tdspp.util .*;

/**
 * The knowledge base that stores the strategy information.
 * 
 * This component is implemented as a collection of rules and associated housekeeping information
 * and methods to manipulate that data.
 * The Rules will be objects of another class Rule. Each rule provides information about the deployment
 * and its outcome. This may be implemented as a collection of deployments and the expected results.
 * 
 * <table border=0> 
 * <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td> <td> TDS++ </td>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td> <td> Optional, Passive </td>
 * </table> 
 * 
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 22 Feb 2002
 * 
*/

public class StrategyAnalysisKBImp extends GenericImp {
	/**
	 * Collection of rules
	 *
	*/
	Hashtable rules = new Hashtable ();
	int clock = 0;
	String[] factors;
	Integer[] values;
	
	/**
	 * Constructor
	 * Used for storing some initial rules. Rules are to be applied per region point. 
	 * Rules provide values of possible outcome. Starting value is 50%, which is value
	 * when we don't have enough information. Every entry contirbutes
	 * to the outcome with some value. Everything below 40% is considered to be risky, 
	 * and everything above 60% is recommended.
	 * 
	 */
	public void StrategyAnalysisKBImp(){
		
	}

	public void tick(){
		clock++;
	}
	
	public int getClock(){
		return clock;
	}
	
	public Hashtable getTable(){
		Hashtable res = new Hashtable ();
		res = (Hashtable) this.rules.clone();
		//System.out.println ("Inside get: size of rules = "+rules.size());
		return res;
	}
	
	public void setTable(Hashtable rules){
		this.rules = (Hashtable) rules.clone();
	}
	
	void enumerate(){
		int i=0;
		factors = new String[rules.size()];
		values = new Integer [rules.size()];
		for(Enumeration e = rules.keys(), v = rules.elements(); e.hasMoreElements() && v.hasMoreElements(); ) {
			factors[i] = (String) e.nextElement();
			values[i] = new Integer (0);
			values[i] = (Integer) v.nextElement();
			i++;
		}
		//System.out.println("No of rules = " +i);
	}
	
	/**
	 * get function to get factors
	 */
	public String[] getFactors() {
		enumerate();
		return factors;
	}
	
	/**
	 * get function to get values
	 */
	public Integer[] getValues() {
		enumerate();
		Integer [] resValues = new Integer [rules.size()];
		resValues = (Integer[]) values.clone();
		return resValues;
	}
	
	
	public void start(){
		/*
		rules.put(MessageConstants.RULE_TERRAINTYPE_SWAMP, new Integer (-30));
		rules.put(MessageConstants.RULE_TERRAINTYPE_RIVER, new Integer (-40));
		rules.put(MessageConstants.RULE_TERRAINTYPE_MOUNTAIN , new Integer (+30));
		rules.put(MessageConstants.RULE_TERRAINTYPE_FLAT , new Integer (+10));
		
		rules.put(MessageConstants.RULE_ARMYRESOURCES_ENEMYINFT , new Integer (-20));
		rules.put(MessageConstants.RULE_ARMYRESOURCES_ENEMYTANK , new Integer (-30));
		rules.put(MessageConstants.RULE_ARMYRESOURCES_MINEFIELD , new Integer (-50));
		rules.put(MessageConstants.RULE_ARMYRESOURCES_INFANTRY , new Integer (+20));
		rules.put(MessageConstants.RULE_ARMYRESOURCES_TANKFIELD , new Integer (+30));
		
		*/
	}
		
	public void populate(String[] factors, Integer[] values, int factNum){
		
		for (int i=0; i< factNum; i++){
			
			rules.put(factors[i], values[i]);
		}
		//System.out.println ("size of rules is " + rules.size());
	}
	
	/**
	 * Store a particular rule in the KB.
	 * Called by: SAKBUI.
	 * 
	 * @param factor	The factor to be inserted.
	 * @param value		The contribution of the factor.
	 * @see		SAKBUIImp#storeRule
	 * @return		void
	 * 
	*/
	public void storeRule(String factor, int value) {
		rules.put (factor, new Integer (value));
		//containsKey
	}
	public void storeRule(String factor, Integer value) {
		rules.put (factor, value);
		//containsKey
	}

	/**
	 * Delete a particular rule from the KB.
	 * Called by: SAKBUI.
	 * 
	 * @param factor	The factor to be deleted.
	 * @see		SAKBUIImp#deleteRule
	 * @return		void
	 * 
	*/
	public void deleteRule(String factor) {
		if (rules.containsKey(factor)) rules.remove (factor);
	}

	/**
	 * Update a particular rule in the KB.
	 * Called by: SAKBUI.
	 * 
	 * @param factor	The factor to be Updated.
	 * @param value		The contribution of the factor.
	 * @see		SAKBUIImp#updateRule
	 * @return		void
	 * 
	*/
	public void updateRule(String factor, int value) {
		
		rules.put (factor, new Integer(value));
	}
	public void updateRule(String factor, Integer value) {
		System.out.println("new value is "+value.toString());
		rules.put (factor, value);
	}
	
}
