package tdspp.WeatherAnalyzer;

import Prism.core.*;
import tdspp.util.*;

/**
 * Analyzes the weather information.
 *
 * This component makes use of the weather information provided by the Weather component
 * and makes the decisions based on that. Thus this acts just as an interface to Weather.
 * 
 * <table border=0> 
 * <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td> <td> TDS++ </td>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td> <td> Variant, Passive </td>
 * </table> 
 * 
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 18 Feb 2002
 * 
*/

public class WeatherAnalyzerImp extends GenericImp {

	/**
	 * Weight given to temperature to calculate the difficulty level
	 */
	int wtTemperature;
	
	/**
	 * Weight given to humidity to calculate the difficulty level
	 */
	int wtHumidity;

	/**
	 * Weight given to windSpeed to calculate the difficulty level
	 */
	int wtWindSpeed;
	
	/**
	 * Weight given to sunlight to calculate the difficulty level
	 */
	int wtSunlight;
	
	/**
	 * Multiplication factor to scale the result to 0-255 scale.
	 */
	int multFactor;

	/**
	 * Constructor
	 * 
	*/
	public WeatherAnalyzerImp(AbstractImplementation parent) {
		super(parent);
	}

	/**
	 * Set the weights and multiplication factor
	 * 
	*/
	public void setParams(int wtTemp, int wtHumid, int wtWind, int wtSun, int mulFact) {
		this.wtTemperature = wtTemp;
		this.wtHumidity = wtHumid;
		this.wtWindSpeed = wtWind;
		this.wtSunlight = wtSun;
		this.multFactor = mulFact;
	}

	/**
	 * The level of difficulty weather may pose for a particular region. It gets the 
	 * weather information from the Weather component and anlyzes that for that difficulties.
	 * Called by : Rendering agent / Analysis agent / Deployment agent
	 * 
	 * @param wi The weather info
	 * @see 	WeatherImp#getWeather
	 * @return	Difficulty level
	 * 
	*/
	public int getWeatherLevel(WeatherInfo wi) {
		long value = 0;
		value += this.wtTemperature * wi.temperature;
		value += this.wtHumidity * wi.humidity;
		value += this.wtWindSpeed * wi.windSpeed;
		value += this.wtSunlight * wi.sunlight;
		value /= this.multFactor;
		if(value < 0) value = 0;
		if(value > 255) value = 255;
		return (int) value;
	}
}
