package tdspp.util;

import java.util.*;
import java.io.*;

/**
 * The Region class represents a region on the map. The map is divided into submaps.
 * Each submap is divided into cells and each cell has information associated with it.
 * The class Region represents a cell.
 *
 * It contains all the information needed for a cell including used resources and conditions there.
 * 
 * <table border=0> 
 * <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td> <td> TDS++ </td>
 * <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td> <td> Utility </td>
 * </table> 
 * 
 * @author 	TDS++ team, USC-CSE
 * @version 	1.0 , 23 May 2002
 * 
*/

public class Region implements Cloneable, Serializable {
	/**
	 * The x and y number of the cell in its submap. The number 0,0 represents the top left corner
	 * of the screen.
	 *
	*/
	public int x,y;

	/**
	 * The submap to which this region belongs.
	 *
	*/
	public int submapId;

	/**
	 * Weather information for the cell.
	 *
	*/
	public WeatherInfo weatherInfo;

	/**
	 * Army resources used by the cell.
	 *
	*/
	public ArmyResources armyRes;

	/**
	 * Terrain description of the cell.
	 *
	*/
	public TerrainDesc terrainDesc;
	
	/**
	 * Constructor
	 * 
	 */
    public Region() {
		this.x = -1;
		this.y = -1;
		this.submapId = -1;
		this.weatherInfo = new WeatherInfo();
		this.armyRes = new ArmyResources();
		this.terrainDesc = new TerrainDesc();
	}

	/**
	 * cloning function
	 */
	public Object clone() {
		Region o = null;
		try {
			o = (Region) super.clone();
		} catch(CloneNotSupportedException e) {
			e.printStackTrace(System.err);
		}
		if(o.weatherInfo != null) o.weatherInfo = (WeatherInfo) o.weatherInfo.clone();
		if(o.armyRes != null) o.armyRes = (ArmyResources) o.armyRes.clone();
		if(o.terrainDesc != null) o.terrainDesc = (TerrainDesc) o.terrainDesc.clone();
		return o;
	}

	public String toString(){
		return "X : " + x + " Y : " + y + "Sub Map ID : " + submapId + "Weather Info"+ weatherInfo.toString() +
			"Army Resources"+ armyRes.toString() + "Terrain Desc" + terrainDesc.toString();
	}
	
	public String toXML(){
		String output = "";
		output += "<Region type=class>";
		output += "<x type=Integer>" + x + "</x>";
		output += "<y type=Integer>" + y + "</y>";
		output += "<submapId type=Integer>" + submapId + "</subMapId>";
		output += weatherInfo.toXML();
		output += armyRes.toXML();
		output += terrainDesc.toXML();
		output += "</Region>";
		return output;
	}
}
